/*
 * Type.c Version 2.0 
 *
 *  See boxFondATMStrike and bBox.h for FONT bounding box from FOND example.
 */


/* try to be compiler independent	*/

#define int16   short   	/* same as int in LSC */
#define card16  unsigned short
#define int32   long    	/* same as int in MPWC */
#define card32  unsigned long

/* maybe we'll check for the functionality */
#ifndef _WaitNextEvent
#define _WaitNextEvent 0xA860
#endif

#ifndef _Unimplemented
#define _Unimplemented 0xA89F
#endif

#define NULL 0L

/*
 * Resource ID constants.
 */

#define appleMenuID	128   	/* This identifies the apple menu*/
#define fileMenuID	129
#define editMenuID	130
#define fontMenuID	131
#define bBoxMenuID  132		/* For the bounding box example  */

#define firstMenuID	appleMenuID
#define menuCount	5

#define appleMenu 		0  /* explicit MyMenus[] array index */
#define aboutMeCommand  1  /* menu item index */

#define fileMenu 		1  /* explicit MyMenus[] array index */
#define quitCommand  	1

#define editMenu 		2  /* explicit MyMenus[] array index */
#define fontMenu 		3  /* explicit MyMenus[] array index */

/* see bBox.h for Bounding box menu and dialog defs. */

/*
 * Dialog stuff
 */
#define	aboutMeDLOG		128

#define	okButton		1
#define cancelButton	2

/* items in the bounding box dialog */

/* alert dialogs to tell user what is happening */
#define noWaitNextEvent 128
#define noATM 130

#include "ATMInterface.h"
#include <color.h>

#include "bBox.h"			/* Bounding box stuff */

int16 main(void);
int16 initMacintosh(void);
int16 initATM(void);
void SetupMenus(void);
int16 InitFontMenu(void);
int16 handleEvent(void);
int16 drawType(void);
Boolean TrapImplemented(int16);
Boolean TrapAvailable(int16, int16);

static float random(void);
static void DoKeyDown(EventRecord *);
static void doCommand(int16,int16);
static void showAboutMeDialog(void) ;

#define _fixedtointround(f) (((f) + 0x00008000L) >> 16)

/* globals  */
MenuHandle *MyMenus;
static 	card32 seed;
int16 	fontitem;
char 	fontname[256];
WindowPtr myWindow;
Boolean hasCQD;

int16 main()
	{
	CursHandle watchHdl;
	seed = 0L;
	if (!initMacintosh())
		return 0;

/* check if ATM is available, quit if not	*/

	if (!initATM())
		{
			Alert(noATM,NULL);
			return 0;
		}

/*   we are going to do some work so let's use the watch */

  	watchHdl = GetCursor(watchCursor);
	SetCursor(*watchHdl);

/* 	setup the font menu so that only available fonts appear undimmed  */
/*	InitFontMenu returns 0 if there are no outlines available to ATM	*/	
	if(!InitFontMenu())
		return 0;
		
	InitCursor();
	for (; handleEvent();)
		drawType();
	return 1;
	}

static int16 initMacintosh()
	{
   	SysEnvRec theWorld;
   	
	InitGraf(&thePort);
	InitFonts();
	InitWindows();
	FlushEvents(everyEvent, 0);
  	InitDialogs(NULL);
	TEInit();
	InitCursor();
	SetupMenus();
	SetFractEnable(true);
    SysEnvirons(1, &theWorld);
	hasCQD = theWorld.hasColorQD;
	if(!TrapImplemented(_WaitNextEvent))
		{
			Alert(noWaitNextEvent,NULL);
			return 0;
		}
	return 1;
	}

static int16 handleEvent()
{
	EventRecord e;
	WindowRecord *w;
	long s;
	Rect b;

if (WaitNextEvent(everyEvent, &e, 0L, 0L))
	{
	switch (e.what)
		{
		case mouseDown:
			switch (FindWindow(e.where, &w))
				{
				case inContent:
					if ((WindowRecord *) FrontWindow() != w)
						SelectWindow(w);
					else if (((WindowRecord *) w)->windowKind < 0)
						SystemClick(&e, w);
					break;
				case inDrag:
					b = (**GrayRgn).rgnBBox;
					DragWindow(w, e.where, &b);
					break;
				case inGoAway:
					if (TrackGoAway(w, e.where))
						{
						CloseWindow(w);
						if (!FrontWindow())
							return 0;
						}
					break;
				case inGrow:
					SetRect(&b, 30, 30, 0x7fff, 0x7fff);
					s = GrowWindow(w, e.where, &b);
					if (s)
						{
						SizeWindow(w, LoWord(s), HiWord(s), 1);
						SetPort(w);
						InvalRect(&((GrafPtr) w)->portRect);
						}
					break;
				case inZoomIn:
					if (TrackBox(w, e.where, inZoomIn))
						{
						ZoomWindow(w, inZoomIn, 0);
						SetPort(w);
						InvalRect(&((GrafPtr) w)->portRect);
						}
					break;
				case inZoomOut:
					if (TrackBox(w, e.where, inZoomOut))
						{
						ZoomWindow(w, inZoomOut, 0);
						SetPort(w);
						InvalRect(&((GrafPtr) w)->portRect);
						}
					break;
				case inMenuBar: 
				{
			    	int32 mResult = MenuSelect(e.where);
      						int16 theMenu, theItem;
      						theMenu = HiWord(mResult); 
      						theItem = LoWord(mResult);
					doCommand(theMenu, theItem);
      						break;
      					}
			}
			break;
		
		
		case updateEvt:
			BeginUpdate(e.message);
			EndUpdate(e.message);
			EraseRect(&((GrafPtr) e.message)->portRect);
			break;
		case keyDown:
      		case autoKey:
			DoKeyDown(&e);
			break;
      		default: break;

		}	/* switch */
	}	/* if */
return 1;
}	/* handleEvent */


static int16 drawType()
	{
	Rect b;
	FixedMatrix m;
	float l;
	RGBColor rgb;
	unsigned char c;
	Fixed d[2];
	
	b = FrontWindow()->portRect;
	m.tx = (Fixed) (random() * b.right * 65536.0);
	m.ty = (Fixed) (random() * b.bottom * 65536.0);
	m.a = (Fixed) (random() * b.right * 65536.0) - m.tx;
	m.b = (Fixed) (random() * b.bottom * 65536.0) - m.ty;
#if 0
	m.c = (Fixed) (random() * b.right * 65536.0) - m.tx;
	m.d = (Fixed) (random() * b.bottom * 65536.0) - m.ty;
#else
	m.c = m.b;
	m.d = - m.a;
#endif

	rgb.red = (l = random(), l * l * l * 65536.0);
	rgb.green = (l = random(), l * l * l * 65536.0);
	rgb.blue = (l = random(), l * l * l * 65536.0);
	if(hasCQD)
		RGBForeColor(&rgb);
	else
		ForeColor(blackColor);
	
	c = random() * 223.0 + 32.0;
	if (c == 0x7f	/* DEL */)
		c++;
	if (c == 0xca	/* non-breaking space */)
		c++;

	d[0] =
	d[1] = 0L;
	
	MoveTo(0, 0);
	if (xyshowTextATM((char *) &c, 1, &m, d))
		{
#if 1
		c++;
		ForeColor(blackColor);
		MoveTo(_fixedtointround(m.tx), _fixedtointround(m.ty));
		LineTo(_fixedtointround(m.a + m.tx), _fixedtointround(m.b + m.ty));
		MoveTo(_fixedtointround(m.tx), _fixedtointround(m.ty));
		LineTo(_fixedtointround(m.c + m.tx), _fixedtointround(m.d + m.ty));
#endif
		}

	return 1;
	}

static float random()
	{
	seed = (seed * 9301L + 49297L) % 233280L;
	return (float) seed / (float) 233280L;
	}

void SetupMenus(void) 
	{
	  int16 i;
	  MyMenus = (MenuHandle *) NewPtr((sizeof(MenuHandle)) * menuCount);
	  for (i = 0; i < menuCount; i++) MyMenus[i] = GetMenu(firstMenuID + i);
	  /* The apple menu is special */
	  AddResMenu(MyMenus[appleMenu], (ResType) 'DRVR');
	  AddResMenu(MyMenus[fontMenu], (ResType) 'FONT');
	  AddResMenu(MyMenus[bBoxMenu], (ResType) 'FONT');
	
	  /* Now insert all of the application menus in the menu bar and draw it */
	  for (i = 0; i < menuCount; i++)
	    if (MyMenus[i] != NULL) InsertMenu(MyMenus[i], 0);
	  DrawMenuBar();
	}

int16 InitFontMenu()
{
	Rect b;
	int16 i,nummen,tempnum,userfontnum;
	fontitem = 0;
/* 
	loop over all the items in the Font Menu and see which faces are
	available, first dim those faces which aren't available to ATM,
	
	CAUTION: fontAvailableATM just checks if there is a PostScript Type1
			 outline font file available for the requested family and style.
			 It does NOT parse the font to determine if it is a valid Type 1 font.
			 For information on actually testing the font, see the last 
			 pagrgraph of section 2.3.2 in the ADVANCED TYPE CAPABILITIES
			 USING ADOBE TYPE MANAGER ON THE MACINTOSH document.
*/
	nummen = CountMItems(MyMenus[fontMenu]);
	for (i=1 ; i <= nummen ; i++)
		{ 
			GetItem(MyMenus[fontMenu],i,fontname);
			GetFNum(fontname,&tempnum);
			if(fontAvailableATM(tempnum, 0))
			{
				fontitem = i;
				userfontnum = tempnum;
			}
			else
			{				
				DisableItem(MyMenus[bBoxMenu],i);	
				DisableItem(MyMenus[fontMenu],i);
			};
		}	/* for */

/* be lazy and use the last one found (if there is one) */
		if(fontitem)
		{
			TextFont(0);	/* shouldn't be necessary */
			CheckItem(MyMenus[fontMenu],fontitem ,true);
	  		DrawMenuBar();
			SetRect(&b, 100, 100, 300, 300);
			if(hasCQD)
				myWindow= (WindowPtr) NewCWindow(0L, &b, fontname, 1, documentProc + 8, -1L, 1, 0L);
			else
				myWindow= (WindowPtr) NewWindow(0L, &b, fontname, 1, documentProc + 8, -1L, 1, 0L);
			SetPort(myWindow);
			TextFont(userfontnum);
			
		}
	return fontitem;
 } /* InitFontMenu  */

static void DoKeyDown(myEvent) 
	EventRecord *myEvent; 
{

	char charCode = (char) (myEvent->message & charCodeMask);
	
	if (myEvent->modifiers & cmdKey) 
  	{ /* command key */
		int32 mResult = MenuKey(charCode);
		int16 theMenu = HiWord(mResult); /* This is the resource ID */
		int16 theItem = LoWord(mResult);
		doCommand(theMenu, theItem);
	}
	return ;
}	/* KeyDown */


/*
   Display the About dialog.
*/
static void showAboutMeDialog(void)
{
  GrafPtr savePort;
  int16 itemHit;
  DialogPtr theDialog;

  GetPort(&savePort);
  theDialog = GetNewDialog(aboutMeDLOG, NULL, (WindowPtr) -1);
  SetPort(theDialog);

  do { ModalDialog(NULL, &itemHit); } while (itemHit != okButton);

  CloseDialog(theDialog);

  SetPort(savePort);
  return;
}	/* showAboutMeDialog */



/*
 * Process mouse clicks in menu bar
 */
static void doCommand(theMenu, theItem)
  int16 theMenu, theItem; 
  {
	 char 	*c;
	 FixedMatrix m;
	 int16 	unrendered;
	 int32 	*dummy;
	 float	thePtSize;
	 int16  userfontnum;

/************ fbbox stuff  *****/
	FamRec  		**fond2Hdl;
	Handle			fondHdl;
	fixedBoxATM 	bBox;
	fixed 			fWidth;

	  switch (theMenu) 
	  {
	    case appleMenuID:
	      if (theItem == aboutMeCommand) 
	      {
	        	showAboutMeDialog();
	      }
	      else 
	      {
	        GrafPtr savePort;
	        char daName[256];
	        GetItem(MyMenus[appleMenu], theItem, daName);
	        GetPort(&savePort);
	        (void) OpenDeskAcc(daName);
	        SetPort(savePort);
	      }
	      break;
	
	    case fileMenuID:
	      switch (theItem) 
	      {
	        case quitCommand:
				ExitToShell();
				break;
	        default:
	        	break;
	      }
	      break;
	
	    case editMenuID:
	      /*
	       * Run this through SystemEdit first.
	       * SystemEdit will return FALSE if it's not a system window.
	       */
	      if (SystemEdit(theItem-1)) break;
	
	   
	    case fontMenuID:
	 		CheckItem(MyMenus[fontMenu], fontitem,false);	/* uncheck old item */
			CheckItem(MyMenus[fontMenu], theItem,true);		/* check new item */
	  		DrawMenuBar();
			GetItem(MyMenus[fontMenu],theItem,fontname);
			GetFNum(fontname,&userfontnum);
			TextFont(userfontnum);
			fontitem = theItem;
	    	SetWTitle(myWindow,fontname);
			EraseRect(&myWindow->portRect);
	    break;

	    case bBoxMenuID:
	 		CheckItem(MyMenus[bBoxMenu], bBoxitem,false);	/* uncheck old item */
			CheckItem(MyMenus[bBoxMenu], theItem,true);		/* check new item */
	  		DrawMenuBar();
			GetItem(MyMenus[bBoxMenu],theItem,fontname);
			GetFNum(fontname,&userfontnum);	
			bBoxitem = theItem;
		  	DoBBoxResultDialog(userfontnum,DoPointSizeDialog());		  		   
	    	/* DoBBoxResultDialog never returns, it does an exit */
	    break;

	    default:
	      break;
	    } /*endswitch theMenu*/
	
	  HiliteMenu(0);
	  return;
  }

Boolean  TrapAvailable(tNumber, tType)
    int16    tNumber;
    int16 tType;
{
    
    
    /* Check and see if the trap exists. */
    /* On 64K ROM machines, tType will be ignored. */
    
       return( NGetTrapAddress(tNumber, tType) !=
               GetTrapAddress(_Unimplemented) );
    
}
    
Boolean TrapImplemented(trapnumber)
    
    int16 trapnumber;
{
    
    /* define trap number for old MPW or non-MPW C */
    
       SysEnvRec theWorld; /* used to check if machine has new traps */
    
    /* Since WaitNextEvent and HFSDispatch both have the same trap
       number ($60), we can only call TrapAvailable for WaitNextEvent
       if we are on a machine that supports separate OS and Toolbox
       trap tables. We call SysEnvirons and check if machineType < 0. */
     
       SysEnvirons(1, &theWorld);
    
    /* Even if we got an error from SysEnvirons, the SysEnvirons glue
       has set up machineType. */
    
       if (theWorld.machineType < 0) {
          return(false);
          /* this ROM doesn't have separate trap tables  */
       } else {
          return(TrapAvailable(trapnumber, ToolTrap));
           /* check for trapnumber */
       }
    
}	/* TrapImplemented */

/*************** START FONT BOUNDING BOX CODE *******************/
/* Note: the heart of the bbox code is in proc boxFondATMStrike */

/****************
*	UpdateStyles - Properly highlights & de/activates the style check boxes in 
*	the point size dialog.  
*****************/
void	UpdateStyles(DialogPtr theDialog,int16 action,int16 setThisOff)
{
  int16 	itemHit;
  short 	itemType;	
  Handle	itemHandle;
  Rect		box;
  static int clearCtl = 0;  /* #def */
 
  GetDItem(theDialog,setThisOff,&itemType,&itemHandle,&box);	/* get the handle   	  */  
  SetCtlValue (itemHandle,clearCtl);							/* clear the radio button */
  GetDItem(theDialog,boldStyle,&itemType,&itemHandle,&box);	
  HiliteControl(itemHandle,action);  
  GetDItem(theDialog,italicStyle,&itemType,&itemHandle,&box);	  
  HiliteControl(itemHandle,action);
  GetDItem(theDialog,shadowStyle,&itemType,&itemHandle,&box);	  
  HiliteControl(itemHandle,action);
  GetDItem(theDialog,outlineStyle,&itemType,&itemHandle,&box); 
  HiliteControl(itemHandle,action);  
  
  DrawControls(theDialog);
}


/****************
* DoPointSizeDialog - Put up pointsize dialog for the bounding box. The following GLOBAL
*					  structure is used to keep the style info in:
*	struct styleDeltas
*	{
*		float llx; 
*  		float lly; 
*	 	float urx;
*	  	float ury;
*		int	italicFlag;
*		int	outlineFlag;
*	};
*
*	struct styleDeltas styleData;
*
*	The FontBBox data returned by proc boxFondATMStrike is only for styles where there
*	is a true scalable font available.  For outline or shadow (or when an outline font
*	isn't available for the style and ATM may synthesize one), your application must
*	update the font bounding box to include these variations.
*	For more info, see section 3.1 in TECHNICAL TIPS FOR MACINTOSH APPLICATIONS USING 
*	ADOBE TYPE MANAGER
* 
*	As an example of the calculations for outline,shadow and ATM synthesized bold and
*	italic styles, conditional code (set by the style radio buttons) is included.  NOTE that here, 
*	they are set by the style buttons and are not smart, like they should be in an application.
*	These alterations should only be done when the bold/italic outlines are not 
*	available.  Also, you may wish to expand each side of the bbox by two pixels to 
*	be conservative.
* 
****************/
 
double DoPointSizeDialog(void)
{
  int16 	itemHit;
  DialogPtr theDialog;
  int16 	type;		/* item type */
  Handle	itemHandle;
  Rect		box;
  Str255	theSizeText;
  
  double	pointSize;
  char 		*charPtr;
  int fillCtl = 1;		/* move to define */
  int oldValue;
  float	urx = 0.0;
  float	lly = 0.0;
  float	llx = 0.0;
  float	ury = 0.0;  	  
	DialogRecord drec;
	
  theDialog = GetNewDialog(pointSizeDLOG, &drec, (WindowPtr) -1);
  SetPort(theDialog);
 
  GetDItem(theDialog,sizeField,&type,&itemHandle,&box);	/* get the handle &... */  
  SelIText (theDialog,sizeField,0,15);   				/* default to selected */
  do 
  { 
  	ModalDialog(NULL, &itemHit); 
  	GetDItem(theDialog,itemHit,&type,&itemHandle,&box);	/* get the handle */  
  	
  	switch (itemHit)
  	{
 	  case boldStyle:
  	  case italicStyle:
  	  case outlineStyle:
  	  case shadowStyle:
	  	oldValue = GetCtlValue(itemHandle);
	  	SetCtlValue (itemHandle,1-oldValue);
	    break;	  
	  default:
	    break;	  	   
  	}		  
  } while (itemHit != okButton);
 
  GetDItem(theDialog,sizeField,&type,&itemHandle,&box);		/* get ptr to fract0 field */  
  GetIText(itemHandle,&theSizeText);			/* contains the text of the point size */
  
  GetDItem(theDialog,boldStyle,&type,&itemHandle,&box);
  GetCtlValue(itemHandle);
  if (GetCtlValue(itemHandle))
    {
  	  urx += boldurxDelta;
  	  lly -= boldllyDelta;
  	}
  GetDItem(theDialog,shadowStyle,&type,&itemHandle,&box);
  if (GetCtlValue(itemHandle))
    {
  	  urx += shadowurxDelta;
  	  lly -= shadowllyDelta;
  	}

  GetDItem(theDialog,italicStyle,&type,&itemHandle,&box);   
  styleData.italicFlag = GetCtlValue(itemHandle);
  
  charPtr = PtoCstr((char *) theSizeText);  
  pointSize = atof(charPtr); 
 
  styleData.llx = llx * pointSize;				/* pointSize * shadow and bold deltas */
  styleData.lly = lly * pointSize;
  styleData.urx = urx * pointSize;
  styleData.ury = ury * pointSize;

  GetDItem(theDialog,outlineStyle,&type,&itemHandle,&box); 
  styleData.outlineFlag = GetCtlValue(itemHandle);

  CloseDialog(theDialog);
  return(pointSize);
}

/****************
*  DoBBoxResultDialog - Put up a dialog box for the bounding box results &
*					    calculate & display the results.
*****************/
void DoBBoxResultDialog(int16 fontNumber,float pointSize)
{
  GrafPtr 		savePort;
  int16 		itemHit;
  DialogPtr 	theDialog;
  int16 		type;		/* item type */
  Handle		itemHandle;
  Rect			box;
  Str255		theSizeText; 
  char 			*charPtr;
  float			multiplier; 
  FamRec  		**fond2Hdl;	/* handle to Family Record (fond) */
  Handle		fondHdl;
  fixedBoxATM 	bBox;
  fixed 		fWidth;
 
  Handle	xliHndl;	/* item handles */
  Handle	yliHndl;
  Handle	xgiHndl;
  Handle	ygiHndl;
  Handle	widthiHndl;

  float	xlfloat;	/* hold the floats after converted from fixed */
  float	ylfloat;
  float	xgfloat;
  float	ygfloat;
  float	widthfloat;

  char	xlStr[15];	/* the strings that hold and display the bounding box */		
  char	ylStr[15];	/** adjust these to the appropriate size **/				
  char	xgStr[15];				
  char	ygStr[15];					
  char	widthStr[15];

  fond2Hdl = (FamRec **) GetResource('FOND',fontNumber);	/* get fond handle 	  */
  
  /******** HERE is the actual call to the BBox routine that does all the work!!!! *************/
  
  boxFondATMStrike(fond2Hdl, &bBox, &fWidth);				/* get bbox from fond */

  multiplier = (pointSize/1000/65536.0);/* boxFondATMStrike returns fixed point wrt 1000 pt fnt*/
										/* /65536 converts fixed pt to float	*/
  xlfloat = (bBox.xl * multiplier);
  ylfloat = (bBox.yl * multiplier);
  xgfloat = (bBox.xg * multiplier);
  ygfloat = (bBox.yg * multiplier);
  widthfloat = (fWidth * multiplier);	/* width hasn't been padded */
		
  xlfloat += styleData.llx;				/* add the STYLES information */
  ylfloat += styleData.lly;
  xgfloat += styleData.urx;
  ygfloat += styleData.ury;
	
  if (styleData.italicFlag)    
  {
	xlfloat = ylfloat * italicMultiplier + xlfloat; /* from Advanced Type...ATM on Mac doc*/
	xgfloat = ygfloat * italicMultiplier + xgfloat;
  }
	  	
  if (styleData.outlineFlag) 
  {
	xlfloat -= 2;				/* expand Box by 2 pixels for outline style */
	ylfloat -= 2;
	xgfloat += 2;
	ygfloat += 2;
  }	     
		
  xlfloat -= 2;					/* expand Box by 2 pixels to be safe */
  ylfloat -= 2;
  xgfloat += 2;
  ygfloat += 2;
		
  sprintf(xlStr,"%f",xlfloat); /* convert to string for display...          */	
  sprintf(ylStr,"%f",ylfloat); /* ...You may want to FORMAT the string here */
  sprintf(xgStr,"%f",xgfloat);
  sprintf(ygStr,"%f",ygfloat);
  sprintf(widthStr,"%f",widthfloat);	/* width hasn't been padded */
	
  theDialog = GetNewDialog(bBoxResultDLOG, NULL, (WindowPtr) -1);
	
  GetDItem(theDialog,2,&type,&xliHndl,&box); 	/* Get handles to dialog items */	
  GetDItem(theDialog,3,&type,&yliHndl,&box);	
  GetDItem(theDialog,4,&type,&xgiHndl,&box);	
  GetDItem(theDialog,5,&type,&ygiHndl,&box);	
  GetDItem(theDialog,6,&type,&widthiHndl,&box);	
   				
  SetIText(xliHndl,(Str255 *) CtoPstr(xlStr)); 	/* Set dialog items for display */
  SetIText(yliHndl,(Str255 *) CtoPstr( ylStr));					
  SetIText(xgiHndl,(Str255 *) CtoPstr(xgStr));					
  SetIText(ygiHndl,(Str255 *) CtoPstr( ygStr));					
  SetIText(widthiHndl, CtoPstr( widthStr));

  do { ModalDialog(NULL, &itemHit); } while (itemHit != okButton);   
  CloseDialog(theDialog);
  ExitToShell();
 return;
}	/* DoBBoxResultDialog */


/*************** START the MEAT of the FONT BOUNDING BOX CODE **************/

/*
 * Bounding box of all ffVersion < 2 fonds, i.e. no bbox exists
 */

#define fondVersionsize 2			/* Defaults */
#define defaultWidth 1700			/* you may want to change this */
static fixedBoxATM defaultBox =
	{
	_inttofixed(-300),
	_inttofixed(-300),
	_inttofixed(1700),
	_inttofixed(1100)
	};

/* 
 * Bounding box of the following Symbol characters:
 *		degree, notequal, infinity, plusminus, greaterequal, Mu, partialdiff, 
 *		summation, product, pi, integral, Omega, radical, approxequal, Delta,
 *		divide, lozenge, and apple.
 *
 *	Although these characters are in some screen fonts, they are not in the
 *	outline and aren't included in the FontBBox in the FOND
 *	To get a true representation of FontBBox including these characters,
 *	the FontBBox in the Fond is unioned with the symbolBox
 *
 */
 
static fixedBoxATM symbolBox =
	{
	_inttofixed(2),
	_inttofixed(-25),
	_inttofixed(887),
	_inttofixed(917)
	};

/*
 *	This proc returns a bounding box and a width for the fond handle passed in.
 *	The bounding box returned is the union of that for all styles this FOND includes
 *	If the FOND does not exist or is a version which does not contain the BBox info,
 *	then a default BBox is returned
 *
*/


int boxFondATMStrike(fond, box, width)
	register FamRec **fond;					/* handle to fond */
	fixedBoxATM *box;
	fixed *width;
{
	FontAssocTable *assocTable;
	FontOffsetTable *offsetTable;
	FontBoxTable *boxTable;
	FontStyleTable *styleTable;
	register FontBoxEntry *boxEntry;
	register int i;
	fixedBoxATM bounds;

	*box = defaultBox;		/* set the defaults: you may wish to change these */
	*width = defaultWidth;
	
	if (!fond)
		return 1;
	
	*width = _fixed12tofixed((**fond).ffWidMax) * 1000;		

	/* only version 2 or greater FONDs have the bbox */
	if (!*fond || (**fond).ffVersion < 2)	
		return 1;

	assocTable = (FontAssocTable *)
		((char *) &(**fond).ffVersion + fondVersionsize);  /* assoc table is 2 bytes after version code */
	offsetTable = (FontOffsetTable *)
		(((char *) assocTable) + sizeof(FontAssocTable) + assocTable->length * 
			sizeof(FontAssocEntry));
	
	boxTable = (FontBoxTable *)
		(((char *) offsetTable) + offsetTable->offset[0]); 	 /* offset[0] points to bbox table */
	styleTable = (FontStyleTable *) 			/* needed to chk if Std Mac Encoding is used   */
		((**fond).ffStylOff ? (char *) (*fond) + (**fond).ffStylOff : 0L);
	
	if (styleTable && boxTable->length >= 0)
		{ /* throw out default box and start from scratch */
		setFixedBox(box, 0L, 0L, 0L, 0L);	

		for (i = 0, boxEntry = boxTable->entry; i <= boxTable->length; i++, boxEntry++) 
/*		for (i = 0, boxEntry = boxTable->entry; i <1; i++, boxEntry++) */

/*	CAUTION: You may not want all of the entries unioned.
 *	this code loops over all FontBBox extries in the FOND and unions the result of each
 *	entry with the existing existing box. See Apple LaserWriter Reference chapter 2 for
 *  info on the FontBBox entry structure. You could just look for the Style word of the
 *  one font you want info on.
 *
*/
			{
			setFixedBox(&bounds,
				_fixed12tofixed(boxEntry->xl) * 1000,
				_fixed12tofixed(boxEntry->yl) * 1000,
				_fixed12tofixed(boxEntry->xg) * 1000,
				_fixed12tofixed(boxEntry->yg) * 1000);
			unionFixedBox(box, &bounds);
			}
		}

/*	if font uses standard Mac Encoding then there are symbol characters in the font
 *	and the symbolBox for those characters must be unioned with the existing box
 *
 * CAUTION: if for some reason you want to see the box for just the font & not symbol,
 * comment this out, however, you should normally use this in calculations.
*/

	if (styleTable && (styleTable->class & 0x0002)) /* 0x0002 = Mac Encoding required */
		unionFixedBox(box, &symbolBox);
		
	return 1;
	}
	
